@@ -1,7 +1,7 @@
 // Copyright 2018 Global Phasing Ltd.
 //
-// Chemical component represents a monomer from the Refmac monomer library,
-// or from PDB CCD.
+// ChemComp - chemical component that represents a monomer from Refmac
+// monomer library, or from PDB CCD.
 
 #ifndef GEMMI_CHEMCOMP_HPP_
 #define GEMMI_CHEMCOMP_HPP_
@@ -74,6 +74,7 @@ struct Restraints {
     double value;
     double esd;
     double value_nucleus;
+    double esd_nucleus;
     std::string str() const { return id1.atom + "-" + id2.atom; }
     std::string lexicographic_str() const {
       return Restraints::lexicographic_str(id1.atom, id2.atom);
@@ -255,21 +256,7 @@ struct Restraints {
     return const_cast<Restraints*>(this)->find_chir(ctr, a, b, c);
   }
 
-  double chiral_abs_volume(const Restraints::Chirality& ch) const {
-    double mult = get_bond(ch.id_ctr, ch.id1).value *
-                  get_bond(ch.id_ctr, ch.id2).value *
-                  get_bond(ch.id_ctr, ch.id3).value;
-    double x = 1;
-    double y = 2;
-    for (double a : {get_angle(ch.id1, ch.id_ctr, ch.id2).value,
-                     get_angle(ch.id2, ch.id_ctr, ch.id3).value,
-                     get_angle(ch.id3, ch.id_ctr, ch.id1).value}) {
-      double cosine = a == 90. ? 0. : std::cos(rad(a));
-      x -= cosine * cosine;
-      y *= cosine;
-    }
-    return mult * std::sqrt(x + y);
-  }
+  double chiral_abs_volume(const Restraints::Chirality& ch) const;
 
   std::vector<Plane>::iterator get_plane(const std::string& label) {
     return std::find_if(planes.begin(), planes.end(),
@@ -285,8 +272,31 @@ struct Restraints {
   }
 };
 
-template<typename Restr> double angle_z(double value_rad, const Restr& restr) {
-  return angle_abs_diff(deg(value_rad), restr.value) / restr.esd;
+template<typename Restr>
+double angle_z(double value_rad, const Restr& restr, double full=360.) {
+  return angle_abs_diff(deg(value_rad), restr.value, full) / restr.esd;
+}
+
+inline double chiral_abs_volume(double bond1, double bond2, double bond3,
+                                double angle1, double angle2, double angle3) {
+  double mult = bond1 * bond2 * bond3;
+  double x = 1;
+  double y = 2;
+  for (double a : {angle1, angle2, angle3}) {
+    double cosine = a == 90. ? 0. : std::cos(rad(a));
+    x -= cosine * cosine;
+    y *= cosine;
+  }
+  return mult * std::sqrt(x + y);
+}
+
+inline double Restraints::chiral_abs_volume(const Restraints::Chirality& ch) const {
+  return gemmi::chiral_abs_volume(get_bond(ch.id_ctr, ch.id1).value,
+                                  get_bond(ch.id_ctr, ch.id2).value,
+                                  get_bond(ch.id_ctr, ch.id3).value,
+                                  get_angle(ch.id1, ch.id_ctr, ch.id2).value,
+                                  get_angle(ch.id2, ch.id_ctr, ch.id3).value,
+                                  get_angle(ch.id3, ch.id_ctr, ch.id1).value);
 }
 
 struct ChemComp {
@@ -328,7 +338,7 @@ struct ChemComp {
   int get_atom_index(const std::string& atom_id) const {
     auto it = find_atom(atom_id);
     if (it == atoms.end())
-      fail("Chemical componenent ", name, " has no atom ", atom_id);
+      fail("Chemical component ", name, " has no atom ", atom_id);
     return int(it - atoms.begin());
   }
 
@@ -426,6 +436,7 @@ inline ChiralityType chirality_from_string(const std::string& s) {
     case 'p': return ChiralityType::Positive;
     case 'n': return ChiralityType::Negative;
     case 'b': return ChiralityType::Both;
+    case '.': return ChiralityType::Both;
     default: throw std::out_of_range("Unexpected chirality: " + s);
   }
 }
@@ -457,14 +468,15 @@ inline ChemComp make_chemcomp_from_block(const cif::Block& block_) {
                               "?type", "?value_order",               // 2, 3
                               "?aromatic", "?pdbx_aromatic_flag",    // 4, 5
                               "?value_dist", "?value_dist_esd",      // 6, 7
-                              "?value_dist_nucleus"})) {             // 8
+                              "?value_dist_nucleus", "?value_dist_nucleus_esd"})) { // 8, 9
     bool aromatic_flag = (row.one_of(4, 5)[0] | 0x20) == 'y';
     double dist = row.has(6) ? cif::as_number(row[6]) : NAN;
     double esd = row.has(7) ? cif::as_number(row[7]) : NAN;
     double dist_nucl = row.has(8) ? cif::as_number(row[8]) : NAN;
+    double esd_nucl = row.has(9) ? cif::as_number(row[9]) : NAN;
     cc.rt.bonds.push_back({{1, row.str(0)}, {1, row.str(1)},
                           bond_type_from_string(row.one_of(2, 3)),
-                          aromatic_flag, dist, esd, dist_nucl});
+                          aromatic_flag, dist, esd, dist_nucl, esd_nucl});
   }
   for (auto row : block.find("_chem_comp_angle.",
                              {"atom_id_1", "atom_id_2", "atom_id_3",
