@@ -38,6 +38,7 @@ public:
   bool with_history = true;          // write MTZ history in comments
   bool skip_empty = false;           // skip reflections with no values
   bool enable_UB = false;            // write _diffrn_orient_matrix.UB
+  bool write_staraniso_tensor = true; // write _reflns.pdbx_aniso_B_tensor_*
   bool write_special_marker_for_pdb = false;
   int less_anomalous = 0;            // skip (+)/(-) columns even if in spec
   std::string skip_empty_cols;       // columns used to determine "emptiness"
@@ -66,9 +67,9 @@ public:
       "& SIG{prev}                L pdbx_F_minus_sigma",
       "? FREE|RFREE|FREER|FreeR_flag|R-free-flags I status S",
       "? FWT|2FOFCWT              F pdbx_FWT",
-      "& PHWT|PH2FOFCWT           P pdbx_PHWT",
+      "& PHWT|PH2FOFCWT           P pdbx_PHWT .3f",
       "? DELFWT|FOFCWT            F pdbx_DELFWT",
-      "& DELPHWT|PHDELWT|PHFOFCWT P pdbx_DELPHWT",
+      "& DELPHWT|PHDELWT|PHFOFCWT P pdbx_DELPHWT .3f",
       nullptr
     };
     static const char* unmerged[] = {
@@ -543,10 +544,47 @@ inline bool validate_merged_intensities(Intensities& mi, Intensities& ui,
 
 #define WRITE(...) os.write(buf, gf_snprintf(buf, 255, __VA_ARGS__))
 
+// Reorder eigenvalues and change signs of eigenvectors in the STARANISO way.
+// It minimises the rotation angle from the basis vectors to the eigenvectors,
+// which is equivalent to maximising the trace of the eigenvector matrix.
+inline void reorder_staraniso_eigensystem(Mat33& vectors, double (&values)[3]) {
+  const int8_t permut[6][3] = {{0,1,2}, {1,2,0}, {2,0,1}, {1,0,2}, {2,1,0}, {0,2,1}};
+  const int8_t signs[8][3] = {{1,1,1}, {1,-1,-1}, {-1,1,-1}, {-1,-1,1},
+                              {-1,-1,-1}, {-1,1,1}, {1,-1,1}, {1,1,-1}};
+  double max_trace = -INFINITY;
+  int permut_pos = 0, sign_pos = 0;
+  bool det_neg = std::signbit(vectors.determinant());
+  for (int i = 0; i < 6; ++i) {
+    int jbase = det_neg == (i > 2) ? 0 : 4;
+    const int8_t (&p)[3] = permut[i];
+    for (int j = jbase; j < jbase+4; ++j) {
+      double trace = 0.;
+      for (int k = 0; k < 3; ++k)
+        trace += signs[j][k] * vectors[k][p[k]];
+      if (trace > max_trace) {
+        max_trace = trace;
+        permut_pos = i;
+        sign_pos = j;
+      }
+    }
+  }
+  const int8_t (&p)[3] = permut[permut_pos];
+  double tmp[3];
+  for (int i = 0; i < 3; ++i) {
+    for (int j = 0; j < 3; ++j)
+      tmp[j] = signs[sign_pos][j] * vectors.a[i][p[j]];
+    std::memcpy(vectors.a[i], tmp, sizeof(tmp));
+  }
+  for (int j = 0; j < 3; ++j)
+    tmp[j] = values[p[j]];
+  std::memcpy(values, tmp, sizeof(tmp));
+}
+
 inline void write_staraniso_b_in_mmcif(const SMat33<double>& b,
                                        char* buf, std::ostream& os) {
   double eigenvalues[3];
   Mat33 eigenvectors = eigen_decomposition(b, eigenvalues);
+  reorder_staraniso_eigensystem(eigenvectors, eigenvalues);
   const char* prefix = "\n_reflns.pdbx_aniso_B_tensor_eigen";
   for (int i = 0; i < 3; ++i) {
     double v = std::fabs(eigenvalues[i]) > 1e-4 ? eigenvalues[i] : 0;
@@ -685,7 +723,7 @@ inline void MtzToCif::write_cif(const Mtz& mtz, const Mtz* mtz2,
     write_cell_and_symmetry(cell, rmsds, mtz.spacegroup, buf, os);
   }
 
-  if (staraniso_b)
+  if (write_staraniso_tensor && staraniso_b)
     write_staraniso_b_in_mmcif(*staraniso_b, buf, os);
 
   if (merged)
